package encapsulation.export;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

import encapsulation.metrics.Measurement;

/**
 * ResultsWriter class, represents a writer that can write out metrics results
 * to file
 * @author Janina Voigt
 *
 */
public abstract class ResultsWriter {
	
	protected PrintWriter writer;
	
	/**
	 * Constructor for ResultsWriter objects
	 * @param fileName the name of the file to write metrics data to
	 */
	public ResultsWriter(String fileName) {
		File file = new File(fileName);
		
		try {
			file.createNewFile();
			writer = new PrintWriter(new FileOutputStream(file));
		}
		catch(IOException e){}
	}

	/**
	 * Write out a list of metrics results
	 * @param measurements the measurements to write out
	 */
	public abstract void writeOutResults(List<Measurement> measurements);
	
	/**
	 * Write out the header for a set of metrics values
	 * @param header the header to write out
	 */
	public abstract void writeOutHeader(String header);	
}
