package encapsulation.metrics;

import java.util.HashSet;
import java.util.Set;

import encapsulation.model.AccessibleMember;
import encapsulation.model.Entity;
import encapsulation.model.Member;

/**
 * FieldMemberStrategy class, defines a strategy for extracting members for metrics
 * calculations given an entity. This strategy only extracts fields from the 
 * given entity.
 * @author Janina Voigt
 *
 */
public class FieldMemberStrategy extends MemberStrategy {

	/**
	 * Gets the members that should be used in a metric calculation given a 
	 * certain entity
	 * @param entity the entity to get members from
	 * @return the members to use in the metrics calculation
	 */
	public Set<AccessibleMember> getMembers(Entity entity){
		Set<Member> allMembers = entity.getMembers();
		Set<AccessibleMember> members = new HashSet<AccessibleMember>();
		for(Member member : allMembers){
			if(member.isField()){
				members.add(member.asAccessibleMember());
			}
		}
		return members;
	}
	
	/**
	 * Gets a header to describe this certain metric
	 * @param entity the entity used in the metric calculation
	 * @return a header to print
	 */
	public String getHeader(Entity entity){
		return "Field information for " + entity.getName();
	}
}
