package jst.symtab.link;

import parser.parsetree.SyntaxTreeLeaf;
import jst.app.JSTMessage;
import jst.symtab.*;

public class AmbiguousLookup extends CompoundLookup {

	protected VariableLookup variableLookup;

	public AmbiguousLookup(ScopeLink qualifier, Identifier name) {
		super(qualifier, name);

		variableLookup = new VariableLookup(qualifier, name);
	}

	public AmbiguousLookup(Decl newOwner, AmbiguousLookup that) {
		super(newOwner, that);

		variableLookup = that.variableLookup.copy(newOwner);
	}

	public AmbiguousLookup copy(Decl newOwner) {
		return new AmbiguousLookup(newOwner, this);
	}

	public boolean resolve() {
		return variableLookup.resolve() || super.resolve();
	}

	public JSTMessage getError() {
		return JSTMessage.PACKAGE_OR_TYPE_OR_VARIABLE_NOT_FOUND;
	}

	public Decl getDecl() {
		if (getVariable() != null)
			return getVariable();

		return super.getDecl();
	}

	public Scope getScope() {
		if (getVariable() != null){
			if(getVariable().getType() == null) {
				return null;
			}
			return getVariable().getType().asReferenceType();
		}

		return super.getScope();
	}

	public VariableDecl getVariable() {
		return variableLookup.getVariableDecl();
	}

	public boolean isInstance() {
		return variableLookup.isInstance();
	}
	
	public SyntaxTreeLeaf getSource() {
		return variableLookup.getSource();
	}
}
