package jst.symtab.link;

import jst.app.JSTMessage;
import parser.parsetree.*;
import jst.parsetree.*;
import jst.symtab.*;

public class BuiltInTypeLink extends ScopeLink implements TypeLink {

	public static BuiltInTypeLink getBoolean(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.BooleanType);
	}

	public static BuiltInTypeLink getVoid(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.VoidType);
	}

	public static BuiltInTypeLink getByte(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.ByteType);
	}

	public static BuiltInTypeLink getShort(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.ShortType);
	}

	public static BuiltInTypeLink getInt(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.IntType);
	}

	public static BuiltInTypeLink getLong(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.LongType);
	}

	public static BuiltInTypeLink getChar(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.CharType);
	}

	public static BuiltInTypeLink getFloat(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.FloatType);
	}

	public static BuiltInTypeLink getDouble(Identifier name) {
		return new BuiltInTypeLink(name, PrimitiveType.DoubleType);
	}

	protected Identifier typeName;

	protected TypeDecl type;  // May differ from name because of promotions.

	protected BuiltInTypeLink(Identifier name, TypeDecl theType) {
		assert theType != null;

		typeName = name;
		type = theType;
	}

	public BuiltInTypeLink(SyntaxTreeLeaf literal) {
		typeName = new StringIdentifier(literal.getValue(), literal.getLine(), literal.getColumn());

		// Temporary hack!
		if (typeName.toString().equals("null"))
        	type = NullType.getNullType();
		else
			type= PrimitiveType.IntType;
		// ToDo: figure out type properly!
	}

	public BuiltInTypeLink copy(Decl newOwner) {
		return this;
	}

	public BuiltInTypeLink copyType(Decl newOwner) {
		return this;
	}

	public String getName() {
		return typeName.getName();
	}


	public int getLine() {
		return typeName.getLine();
	}

	public int getColumn() {
		return typeName.getColumn();
	}

	public String getOwnerFileName() {
		return "dunno";
	}

	public String getFullName() {
		return typeName.getName();
	}

	public boolean resolve() {
		return true;
	}

	public JSTMessage getMessage() {
		return null;
	}

	public Decl getDecl() {
		return getType();
	}

	public TypeDecl getType() {
		return type;
	}

	public void unaryNumericPromotion() {

		//If the operand is of compile-time type byte, short, or char, unary numeric promotion
        // promotes it to a value of type int by a widening conversion (#5.1.2).
        if (type == PrimitiveType.ByteType || type == PrimitiveType.DoubleType)
        	type = PrimitiveType.DoubleType;

		// Otherwise, a unary numeric operand remains as is and is not converted.
	}

	public void binaryNumericPromotion(BuiltInTypeLink that) {

		TypeDecl thatType = that.getType();

    	// When an operator applies binary numeric promotion to a pair of operands, each of which
        // must denote a value of a numeric type, the following rules apply, in order, using
        // widening conversion (#5.1.2) to convert operands as necessary:

        //   If either operand is of type double, the other is converted to double.
        if (type == PrimitiveType.DoubleType || thatType == PrimitiveType.DoubleType)
        	type = PrimitiveType.DoubleType;

		//   Otherwise, if either operand is of type float, the other is converted to float.
		else if (type == PrimitiveType.FloatType || thatType == PrimitiveType.FloatType)
        	type = PrimitiveType.FloatType;

		//   Otherwise, if either operand is of type long, the other is converted to long.
		else if (type == PrimitiveType.LongType || thatType == PrimitiveType.LongType)
        	type = PrimitiveType.LongType;

		//   Otherwise, both operands are converted to type int.
		else
			type = PrimitiveType.IntType;
	}

	public boolean isBoolean() {
		return type == PrimitiveType.BooleanType;
	}

	public SyntaxTreeLeaf getSource() {
		return null;
	}

	public String toString() {
		return type.toString();
	}
	
	public boolean isInstance() {
		return true;
	}
	
	protected Scope getOriginScope() {
		// No way to work this out!
		// return arrayTypeLink.getOriginScope();
		return PackageDecl.getDefaultPackage(); // Just for now...
	}

	public Scope getScope() {
		//This is really ugly!
		//We should check if the type is a String and make it a scope
		//otherwise return null
		//works for now
		return getType() == null? null : getType().asScope();
	}

	public boolean isFirst() {
		return false;
	}
}
