package jst.symtab.link;

import java.util.*;
import jst.app.JSTMessage;
import jst.symtab.*;

public class ConstructorLookup extends OperationLookup {
	
	// Syntax allows interfaces.  Eliminate them semantically.
	protected UserTypeLookup userTypeLookup;

	protected ConstructorDecl constructor;

	/*
	public ConstructorLookup(Link theQualifier, Identifier name) {
		super(theQualifier, name);

		classLookup = new ClassTypeLookup(qualifier, id);
	}
	*/

	public ConstructorLookup(UserTypeLookup theUserTypeLookup) {
		super(theUserTypeLookup.qualifier, theUserTypeLookup.id);

		userTypeLookup = theUserTypeLookup;
	}

	public ConstructorLookup(Decl newOwner, ConstructorLookup that) {
		super(newOwner, that);

		userTypeLookup = that.userTypeLookup.copy(newOwner);
	}

	public ConstructorLookup copy(Decl newOwner) {
		return new ConstructorLookup(newOwner, this);
	}

	public ConstructorLookup copyType(Decl newOwner) {
		return copy(newOwner);
	}

	public boolean resolve() {
		// return classLookup.resolve() && lookup() && consistentContext();
		return userTypeLookup.resolve() && lookup();
	}

	public boolean lookup() {

		if (!resolveTypeArgs())
			return false;

		if (!resolveArgs())
			return false;

		UserType type = userTypeLookup.getUserType();
		ClassType classType = type.asClass();
		if (classType == null)
			return false;

		constructor = classType.lookupConstructor(/*getTypeArgs(),*/ getArgTypes(), getLooker());

		return constructor != null;
	}

	protected JSTMessage getError() {
		return JSTMessage.CONSTRUCTOR_NOT_FOUND;
	}

	public OperationDecl getOperationDecl() {
		return getConstructorDecl();
	}

	public ConstructorDecl getConstructorDecl() {
		return constructor;
	}

	// This is the only time we can move from static to instance context.
	protected boolean consistentContext() {
		return true;
	}

	public boolean isInstance() {
		return true;
	}
}
