package jst.symtab.link;

import parser.parsetree.SyntaxTreeLeaf;
import jst.symtab.*;
import jst.app.JSTMessage;

public abstract class IdLookup extends Lookup {
	
	protected ScopeLink qualifier;

	protected Identifier id;

	protected IdLookup(ScopeLink theQualifier, Identifier name) {
		qualifier = theQualifier;
		id = name;

		assert qualifier != null;
		assert id != null;
	}

	public IdLookup(Decl newOwner, IdLookup that) {
		qualifier = that.qualifier.copy(newOwner);
		id = that.id;
	}
	
	public ScopeLink getQualifier() {
		return qualifier;
	}

	public String getName() {
		return id.getName();
	}

	public int getLine() {
		return id.getLine();
	}

	public int getColumn() {
		return id.getColumn();
	}

	public String getOwnerFileName() {
		return qualifier.getOwnerFileName();
	}

	public String getFullName() {
		if (qualifier.isFirst())
			return getName();

		return qualifier.getFullName() + "." + getName();
	}

	public boolean isFirst() {
		return false;
	}

	public boolean resolve() {
		if (!qualifier.resolve())
		   return false;
	
		if (!lookup()) {
			if (!reflect())
				return false;

			if (!lookup())
				return false;
		}

		return consistentContext();
	}

	protected abstract boolean lookup();

	protected boolean reflect() {
		return false;
	}

	protected boolean widen() {
		return qualifier.isFirst();
	}

	// Can't move from static to instance context.
	protected boolean consistentContext() {
		return qualifier.isInstance() || !isInstance();
	}

	protected Scope getOriginScope() {
		return qualifier.getOriginScope();
	}

	public JSTMessage getMessage() {
		if (qualifier.getDecl() == null)
			return qualifier.getMessage();

		return super.getMessage();
	}
	
	public SyntaxTreeLeaf getSource() {
		return id.getSource();
	}
}
