package parser.pda;

import parser.parsetree.*;
import java.util.*;

public class RekersStackNode extends IntStackNode {
    /** @associates parser.pda.RekersStackNode
     * @supplierRole previous
     * @supplierCardinality 0..**/
    public Map previous; // RekersStackNode ==> SyntaxTreeNode

    // Make a node with no previous node.
    public RekersStackNode(int theState) {
        state = theState;
        previous = new HashMap();
    }

    public RekersStackNode(RekersStackNode aPrevious, SyntaxTree tree, int theState) {
        this(theState);
        addLink(aPrevious, tree);
    }

    public void addLink(RekersStackNode aPrevious, SyntaxTree tree) {
        Object o = previous.put(aPrevious, tree);
        assert o == null : "Attempt to reassign previous for a node.";
    }

    public boolean hasLink(RekersStackNode aPrevious) {
        return previous.containsKey(aPrevious);
    }

    // RekersStackNode ==> SyntaxTreeNode
    public Map getPrevious() {
        return previous;
    }

    public String dump(String indent, Set done, String[] stateNames) {
        String result = indent + stateNames[state];

        if (done.contains(this)) {
            result += " (above)\n";
            return result;
        }
        else {
            done.add(this);
            result += "\n";
            indent += " ";
        }

        Iterator entryIter = previous.entrySet().iterator();
        while (entryIter.hasNext()) {
            Map.Entry entry = (Map.Entry) entryIter.next();
            RekersStackNode prev = (RekersStackNode) entry.getKey();
            SyntaxTree tree = (SyntaxTree) entry.getValue();

            String value = tree.toString();
            if (value.length() > 50)
                value = value.substring(0, 50);
            result += indent + value + "\n";

            result += prev.dump(indent + " ", done, stateNames);
        }

        return result;
    }
}

