package util;

import java.util.*;

/**
 */
public class TwoWayMultiMap extends MultiMap {

    protected MultiMap inverseContents;

    public TwoWayMultiMap() {
        inverseContents = new MultiMap();
    }

    public TwoWayMultiMap(MultiMap data) {
        inverseContents = data.invert();
    }

	public void clear() {
        super.clear();
    	inverseContents.clear();
    }

    public Set getKeys(Object value) {
        return inverseContents.get(value);
    }

	public boolean containsValue(Object value) {
        return inverseContents.containsKey(value);
    }

	public Set put(Object key, Object value) {
        inverseContents.put(value, key);
        return super.put(key, value);
    }

	public Set put(Object key, Set values) {
        // Might be called with a null value set.
        if (values != null) {
            Iterator valueIter = values.iterator();
            while (valueIter.hasNext()) {
                Object value = valueIter.next();

                inverseContents.put(value, key);
            }
        }
    
        return super.put(key, values);
    }

	public Set remove(Object key) {
        Set values = get(key);

        Iterator valueIter = values.iterator();
        while (valueIter.hasNext()) {
            Object value = valueIter.next();

            inverseContents.remove(value, key);
        }

    	return super.remove(key);
    }

	// Extra method.
	public boolean remove(Object key, Object value) {
        inverseContents.remove(value, key);
        return super.remove(key, value);
    }
}
